#include "application.h"
#include <libgnome/gnome-i18n.h>
#include <gtkmm/menu.h>
#include <gtkmm/menubar.h>
#include <libgnomemm/main.h>
#include <libgnomeuimm/about.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>
#include <gtkmm.h>
#include <cstdio>
#include <sstream>

#include <libbonobo.h>

#include "GNOME_Bluetooth_Manager.h"

#include "gsm.h"

#include "giveusaclue.h"

PMgrApp::PMgrApp()
{
    init();
}

PMgrApp::~PMgrApp()
{
  if (m_prefsmgr) {
	delete m_prefsmgr;
  }
  if (m_btmanager) {
	delete m_btmanager;
  }
  unhook_listener();
}

void
PMgrApp::show_preferences()
{
  // runs on main GUI thread, no ned foro
  // protection
  m_prefsmgr->show();
}

bool
PMgrApp::process_messages()
{
  PhoneMessage *me=0;
  Glib::Mutex::Lock lock(messages_mutex);

  if (!messages.empty() && !m_dialog_visible) {
	me=messages.front();
	messages.pop();
  }
  lock.release();

  if (me && !m_dialog_visible) {
	gdk_threads_enter();
	Gtk::Dialog *dialog=0;
	Gtk::TextView *textview=0;
	Gtk::Label *l_sender, *l_sent=0;

	// set the ui feedback for message
	send_ui_feedback(STATE_MESSAGE,
					 _("Phone Manager has received a message"));

	// get the widgets
	m_ui->get_widget("sms_dialog", dialog);
	m_ui->get_widget("senderlabel", l_sender);
	m_ui->get_widget("datelabel", l_sent);
	m_ui->get_widget("messagecontents", textview);

	// populate them
	Glib::RefPtr<Gtk::TextBuffer> buf=textview->get_buffer();
	buf->set_text(me->getMessage());
	l_sender->set_text(me->getSender());
	l_sent->set_text(me->getTimestamp());

    // send dashboard cluepacket on dialog focus

    dialog->signal_focus_in_event().connect(
            SigC::slot(*this, &PMgrApp::on_message_dialog_focus));

	dialog->show_all();
	m_dialog_visible=true;

	gdk_threads_leave();

	// finally, throw away the message
	delete me;
  } else {
	// snooze a bit
	Glib::usleep(100000);
  }

  return true;
}

void
PMgrApp::create_reply()
{
  Gtk::Label *l_sender;
  m_ui->get_widget("senderlabel", l_sender);
  create_send_dialog(l_sender->get_text());
}

void
PMgrApp::on_dialog_hide()
{
  // if no messages left, clear the message icon
  if (messages.empty())
	send_ui_feedback(STATE_NORMAL, _("Phone Manager is connected to your phone"));

  m_dialog_visible=false;
}

bool
PMgrApp::on_message_dialog_focus(GdkEventFocus *ev)
{
	Gtk::TextView *textview=0;
    Glib::RefPtr<Gtk::TextBuffer> buf;
	Gtk::Label *l_sender, *l_sent=0;
	m_ui->get_widget("senderlabel", l_sender);
	m_ui->get_widget("datelabel", l_sent);
	m_ui->get_widget("messagecontents", textview);
    buf=textview->get_buffer();

    send_dashboard_clue(l_sender->get_label().c_str(),
            buf->get_text().c_str());
    return TRUE;
}

void
PMgrApp::on_phone_message(Glib::ustring sender,
						  Glib::ustring sent,
						  Glib::ustring msg)
{
  Glib::Mutex::Lock lock(messages_mutex);
  PhoneMessage *me=new PhoneMessage(sender, sent, msg);
  messages.push(me);
  lock.release();
}

void
PMgrApp::set_ui_status_hook(PMgrApp::type_feedbackFunc func,
							gpointer userdata=NULL)
{
  m_ui_feedback=func;
  m_ui_feedback_data=userdata;
}

void
PMgrApp::send_ui_feedback(guint status, gchar *tooltip)
{
  if (m_ui_feedback)
	(*m_ui_feedback)(status, tooltip, m_ui_feedback_data);
}

void
PMgrApp::on_status_change(gint status)
{
  // have threads protection as runs on
  // slave thread
  gdk_threads_enter();
  switch(status)
	{
	case PHONELISTENER_CONNECTING:
	  g_message("phone manager connecting");
	  send_ui_feedback(STATE_WORKING,
					   _("Phone Manager is connecting to your phone"));
	  break;
	case PHONELISTENER_CONNECTED:
	  g_message("phone manager connected");
	  send_ui_feedback(STATE_NORMAL,
					   _("Phone Manager is connected to your phone"));
	  break;
	case PHONELISTENER_DISCONNECTING:
	  g_message("phone manager disconnecting");
	  send_ui_feedback(STATE_WORKING,
					   _("Phone Manager is disconnecting from your phone"));
	  break;
	case PHONELISTENER_IDLE:
	  g_message("phone manager idle");
	  send_ui_feedback(STATE_NORMAL,
					   _("Phone Manager"));
	  break;
	case PHONELISTENER_ERROR:
	  g_message("phone manager error");
	  send_ui_feedback(STATE_ERROR,
					   _("Phone Manager is unable to connect to your phone"));
	  break;
	}
  gdk_threads_leave();
}

bool
PMgrApp::restart_listener()
{
  hook_up_listener(m_prefsmgr->connect_port());
  return false;
}

void
PMgrApp::schedule_retry()
{
  // set a timeout for a retry in 10 seconds
  if (m_prefsmgr->auto_retry())
	m_timeout_conn=Glib::signal_timeout().connect(SigC::slot(*this,
															 &PMgrApp::restart_listener),
												  10000);
}

void
PMgrApp::on_listener_exit()
{
  if (!m_requested_listener_death) {
	// this was an unexpected death
	// first, we'll clean up
	unhook_listener();
	// then we'll schedule a restart if necessary
	schedule_retry();
  }
}

void
PMgrApp::init()
{
  m_ui=Gnome::Glade::Xml::create(UIDIR"phonemgr.glade");
  m_ui_feedback=NULL;

  // message dialog stuff
  Gtk::Dialog *dialog=0;
  Gtk::Button *but=0;

  m_dialog_visible=false;

  m_ui->get_widget("sms_dialog", dialog);
  m_ui->get_widget("okbutton", but);
  // connect signals
  // make sure the OK button causes the dialog to disappear
  but->signal_clicked().connect(SigC::slot(*dialog, &Gtk::Widget::hide));

  m_ui->get_widget("replybutton", but);
  but->signal_clicked().connect(SigC::slot(*this, &PMgrApp::create_reply));


  dialog->signal_hide().connect(SigC::slot(*this, &PMgrApp::on_dialog_hide));

  // hook up notification for when slave thread dies
  m_dispatcher=new Glib::Dispatcher();
  m_dispatcher->connect(SigC::slot(*this, &PMgrApp::on_listener_exit));

  m_btmanager=new BTManager();
  m_prefsmgr=new PrefsManager(this, m_ui);

  m_prefsmgr->signal_serial().connect(SigC::slot(*this,
												 &PMgrApp::on_port_change));

  // idle handler to pick up messages and display them
  Glib::signal_idle().connect(SigC::slot(*this, &PMgrApp::process_messages));

  // hook up and connect the SMS listener
  hook_up_listener(m_prefsmgr->connect_port());

  // TODO check for slave creation failure
}

bool
PMgrApp::hook_up_listener(const Glib::ustring newport)
{
  // if a timeout for reconnect is scheduled then switch it off
  // as we don't want two reconnects or more to happen simultaneously
  if (m_timeout_conn.connected()) {
	m_timeout_conn.disconnect();
  }

  m_requested_listener_death=false;
  m_pl=new PhoneListener();
  m_msg_conn=m_pl->signal_notify().connect(SigC::slot(*this,
										   &PMgrApp::on_phone_message));
  m_status_conn=m_pl->signal_status().connect(SigC::slot(*this,
										 &PMgrApp::on_status_change));

  if (newport!="") {
	m_slave_thread=m_pl->sms_loop_thread(newport, m_dispatcher);
  } else {
	// the new port is empty string if the Bluetooth manager
	// failed to allocate a serial port.  This may well be because
	// the phone's out of range.  So we schedule a retry if requested
	// so we can hook up to the phone when it does come into range.
	m_msg_conn.disconnect();
	m_status_conn.disconnect();
	m_slave_thread=NULL;
	delete m_pl;
	schedule_retry();
  }
  if (m_slave_thread)
	return true;
  else
	return false;
}

void
PMgrApp::unhook_listener()
{
  if (m_slave_thread)
  {
	m_requested_listener_death=true;
	// disconnect signal handlers
	m_msg_conn.disconnect();
	m_status_conn.disconnect();

	m_pl->stop();
	m_slave_thread->join();
	m_slave_thread=0;
	delete m_pl;
  }
}

void
PMgrApp::on_port_change(const Glib::ustring newport)
{
  // gconf notification
  g_message("serial port changed to %s",
			newport.c_str());
  g_message("killing listener");
  if (m_slave_thread) {
	unhook_listener();
  }
  g_message("creating new listener");
  hook_up_listener(newport);
}

bool
PMgrApp::get_bluetooth_active()
{
  return m_btmanager->get_bluetooth_active();
}

void
PMgrApp::create_send_dialog(Glib::ustring recip)
{
  SendDialog *s;
  s=new SendDialog(recip);

  s->signal_message().connect(SigC::slot(*this,
										 &PMgrApp::queue_new_message));
}

void
PMgrApp::queue_new_message(Glib::ustring recip,
						   Glib::ustring message)
{
  m_pl->queue_outgoing_message(recip, message);
}

// SendDialog

SendDialog::SendDialog()
{
  init("");
}

SendDialog::SendDialog(Glib::ustring recip)
{
  init(recip);
}

SendDialog::~SendDialog()
{
}

static void
destroy_send_dialog(SendDialog *s)
{
  delete s;
}

void
SendDialog::init(Glib::ustring recip)
{
  Gtk::Button *but=0;

  m_ui=Gnome::Glade::Xml::create(UIDIR"phonemgr.glade",
								 "send_dialog");

  // initialise cached pointers for class memebrs
  m_ui->get_widget("send_dialog", m_dialog);
  m_ui->get_widget("recipient", m_entry);
  m_ui->get_widget("messagebody", m_msgbody);
  m_ui->get_widget("charsleft", m_charsleft);

  m_buf=m_msgbody->get_buffer();

  // hook up button signals
  m_ui->get_widget("msgcancelbutton", but);
  but->signal_clicked().connect(SigC::slot(*m_dialog,
										   &Gtk::Widget::hide));
  m_ui->get_widget("sendbutton", m_sendbutton);
  m_sendbutton->signal_clicked().connect(SigC::slot(*this,
													&SendDialog::send));

  // set text defaults
  m_entry->set_text(recip);
  m_buf->set_text("");
  m_sendbutton->set_sensitive(false);

  // hook up action for typing
  m_buf->signal_changed().connect(SigC::slot(*this,
											 &SendDialog::buffer_changed));
  m_entry->signal_changed().connect(SigC::slot(*this,
											   &SendDialog::buffer_changed));

  // program ourself to be deleted on dialog hide
  m_dialog->signal_hide().connect(SigC::bind<SendDialog*>
								  (SigC::slot(&destroy_send_dialog),
								   this));

  // show dialog, get going
  m_dialog->show();
}

void
SendDialog::buffer_changed()
{
  int i=m_buf->get_char_count();

  m_sendbutton->set_sensitive(i>0 && i<=160 &&
							  m_entry->get_text_length()>0);

  if (i>MAX_MESSAGE_LENGTH) {
	m_charsleft->set_text(_("Message too long!"));
  } else {
	std::ostringstream cnum;

	cnum << (MAX_MESSAGE_LENGTH-i);
	m_charsleft->set_text(Glib::locale_to_utf8(cnum.str()));
  }
}


/**
 * called when send button pressed on send dialog
 */
void
SendDialog::send()
{
  m_signal_message.emit(m_entry->get_text(),
						m_buf->get_text());

  // hide ourselves: this also destroys us!
  m_dialog->hide();
}

/**
 * signal to emit when sending a message
 */
SendDialog::type_signal_message
SendDialog::signal_message()
{
  return m_signal_message;
}

// PhoneMessage class implementations

PhoneMessage::PhoneMessage(Glib::ustring sender,
						   Glib::ustring sent,
						   Glib::ustring msg)
{
  _message=msg;
  _sender=sender;
  _sent=sent;
}

Glib::ustring
PhoneMessage::getSender()
{
  return _sender;
}

Glib::ustring
PhoneMessage::getTimestamp()
{
  return _sent;
}

Glib::ustring
PhoneMessage::getMessage()
{
  return _message;
}
