#ifndef _APPLICATION_H
#define _APPLICATION_H

#include <gtkmm.h>
#include <libgnomeuimm/app.h>
#include <libgnomeuimm/appbar.h>
#include <libgnomeuimm/about.h>
#include <libglademm.h>
#include <gconfmm.h>
#include <vector>
#include <queue>
#include <list>

#include "gsm.h"
#include "prefs.h"
#include "btmanager.h"


enum {
  STATE_NORMAL,
  STATE_WORKING,
  STATE_MESSAGE,
  STATE_ERROR,
  STATE_LAST
};

#define MAX_MESSAGE_LENGTH 160

class PMgrApp;
class PhoneMessage;
class BluetoothDevice;
class SendDialog;

class PMgrApp : public SigC::Object
{
    public:
       PMgrApp();
       virtual ~PMgrApp();

	   void show_preferences();
	   void set_btmanager(GNOME_Bluetooth_Manager btm);
	   bool get_bluetooth_active();

	   void create_send_dialog(Glib::ustring recip);
	   // signal handlers
	   virtual void on_phone_message(Glib::ustring sender, 
									 Glib::ustring sent,
									 Glib::ustring msg);
	   virtual void on_status_change(gint status);

	   void on_port_change(Glib::ustring newport);

	   typedef void (*type_feedbackFunc)(guint, gchar *, gpointer);
	   void set_ui_status_hook(PMgrApp::type_feedbackFunc func,
							   gpointer userdata);

	   BTManager *m_btmanager;
    protected:
       void init();
	   void on_dialog_hide();
       bool on_message_dialog_focus(GdkEventFocus *ev);
	   bool hook_up_listener(Glib::ustring newport);
	   void unhook_listener();
	   Glib::Mutex messages_mutex;
	   bool process_messages();
	   PhoneListener *m_pl;
	   Glib::RefPtr<Gnome::Glade::Xml> m_ui;
	   Glib::Thread *m_slave_thread;
	   std::queue<PhoneMessage*> messages;
	   Glib::RefPtr<Gnome::Conf::Client> m_refClient;

	   // callback connection IDs from GConf::Client::notify_add()
	   std::list<guint> m_listNotifyIDs; 
	   PrefsManager *m_prefsmgr;
	   SigC::Connection m_msg_conn;
	   SigC::Connection m_status_conn;
	   SigC::Connection m_timeout_conn;
	   type_feedbackFunc m_ui_feedback;
	   gpointer m_ui_feedback_data;
	   void send_ui_feedback(guint status, gchar *tooltip);
	   Glib::Dispatcher *m_dispatcher;
	   bool m_requested_listener_death;
	   void on_listener_exit();
	   bool restart_listener();
	   void schedule_retry();
	   bool m_dialog_visible;
	   void queue_new_message(Glib::ustring r, Glib::ustring m);
	   void create_reply();
};

class SendDialog : public SigC::Object
{
 public:
  SendDialog();
  SendDialog(Glib::ustring recip);
  ~SendDialog();
  void send();
  typedef SigC::Signal2<void, Glib::ustring, Glib::ustring>
	type_signal_message;
  type_signal_message signal_message();

 protected:
  type_signal_message m_signal_message;
  void buffer_changed();

 private:
  void init(Glib::ustring recip);
  Glib::RefPtr<Gnome::Glade::Xml> m_ui;
  Gtk::Dialog *m_dialog;
  Gtk::TextView *m_msgbody;
  Gtk::Label *m_charsleft;
  Glib::RefPtr<Gtk::TextBuffer> m_buf; 
  Gtk::Entry *m_entry;
  Gtk::Button *m_sendbutton;
};

class PhoneMessage
{
 public:
  PhoneMessage(Glib::ustring sender, Glib::ustring sent, Glib::ustring msg);

  Glib::ustring getMessage();
  Glib::ustring getSender();
  Glib::ustring getTimestamp();

 protected:
  Glib::ustring _message;
  Glib::ustring _sender;
  Glib::ustring _sent;
};

#endif
