
#include <list>
#include <sstream>

#include <gtkmm/menuitem.h>

#include <libbonobo.h>

#include "GNOME_Bluetooth_Manager.h"

#include "btmanager.h"

BTManager::BTManager()
{
  set_btmanager(bonobo_get_object("OAFIID:GNOME_Bluetooth_Manager",
								  "GNOME/Bluetooth/Manager", NULL));
  CORBA_exception_init(&m_ev);
}

BTManager::~BTManager()
{
  CORBA_exception_free(&m_ev);
  if (m_bluetooth_active)
	bonobo_object_release_unref(m_btmanager, NULL);
}

void
BTManager::set_btmanager(GNOME_Bluetooth_Manager btm)
{
  m_btmanager=btm;
  
  if (m_btmanager==CORBA_OBJECT_NIL) {
	g_warning(_("Couldn't get instance of BT Manager"));
	m_bluetooth_active=false;
  } else {
	m_bluetooth_active=true;
  }
}

bool
BTManager::get_bluetooth_active() 
{
  return m_bluetooth_active;
}

std::list<BluetoothDeviceItem *> *
BTManager::get_device_list()
{
  std::list<BluetoothDeviceItem *> *l;
  guint i;

  GNOME_Bluetooth_DeviceList *list;
  GNOME_Bluetooth_Manager_knownDevicesOfClass(m_btmanager, 
											  &list,
											  GNOME_Bluetooth_MAJOR_PHONE,
											  &m_ev);
  l=new std::list<BluetoothDeviceItem *>();
  for(i=0; i<list->_length; i++) {
	Glib::ustring label;
	BluetoothDeviceItem *item;
	label = list->_buffer[i].name;
	// << "(" << (list->_buffer[i].bdaddr) << ")";
	item=new BluetoothDeviceItem(label, 
							 list->_buffer[i].name, 
							 list->_buffer[i].bdaddr);
	item->show();
	l->push_back(item);
  }
  CORBA_free(list);

  return l;
}

void
BTManager::update_device_list()
{
  GNOME_Bluetooth_Manager_discoverDevices(m_btmanager, &m_ev);
}

void
BTManager::free_device_list(std::list<BluetoothDeviceItem *> *list)
{
  while(!list->empty()) {
	// BluetoothDeviceItem *item=list->front();
	list->pop_front();
	// TODO: figure out why this crashes things :(
	// delete item;
  }
  delete list;
}

gint
BTManager::get_rfcomm_port(Glib::ustring bdaddr)
{
  CORBA_long port=GNOME_Bluetooth_Manager_connectRFCOMMPortByService
	(m_btmanager,
	 bdaddr.c_str(),
	 GNOME_Bluetooth_DIALUP_NET_SVCLASS_ID,
	 &m_ev);
  return port;
}

BluetoothDeviceItem::BluetoothDeviceItem(const Glib::ustring& label, 
										 Glib::ustring name, Glib::ustring bdaddr)
  :	Gtk::MenuItem(),
	m_btdevice(name, bdaddr)
{
  add_accel_label(label, false);
}

// why does this destructor crash?
BluetoothDeviceItem::~BluetoothDeviceItem() {}

BluetoothDevice &
BluetoothDeviceItem::get_btdevice()
{
  return m_btdevice;
}

BluetoothDevice::BluetoothDevice(Glib::ustring name, Glib::ustring bdaddr)
{
  _name=name;
  _bdaddr=bdaddr;
}

Glib::ustring
BluetoothDevice::get_name()
{
  return _name;
}

Glib::ustring
BluetoothDevice::get_bdaddr()
{
  return _bdaddr;
}
