#ifndef _PHONEMGR_GSM_H
#define _PHONEMGR_GSM_H

#include <glibmm.h>

#include <gsmlib/gsm_me_ta.h>
#include <gsmlib/gsm_event.h>

// local class to handle SMS events

struct IncomingMessage
{
  // used if new message is put into store
  int _index;                   // -1 means message want send directly
  std::string _storeName;
  // used if SMS message was sent directly to TA
  gsmlib::SMSMessageRef _newSMSMessage;
  // used if CB message was sent directly to TA
  gsmlib::CBMessageRef _newCBMessage;
  // used in both cases
  gsmlib::GsmEvent::SMSMessageType _messageType;

  IncomingMessage() : _index(-1) {}
};

struct OutgoingMessage
{
  std::string number;
  std::string message;
};


enum {
  PHONELISTENER_IDLE,
  PHONELISTENER_CONNECTING,
  PHONELISTENER_CONNECTED,
  PHONELISTENER_DISCONNECTING,
  PHONELISTENER_ERROR
};

class EventHandler : public gsmlib::GsmEvent
{
public:
  // inherited from GsmEvent
  void SMSReception(gsmlib::SMSMessageRef newMessage,
                    gsmlib::GsmEvent::SMSMessageType messageType);
  void CBReception(gsmlib::CBMessageRef newMessage);
  void SMSReceptionIndication(std::string storeName, unsigned int index,
                              gsmlib::GsmEvent::SMSMessageType messageType);
  
  virtual ~EventHandler() {}

  // extended
  void setStoreName(std::string rs);
  void setMessageQueue(vector<IncomingMessage> *nm);
protected:
  string receiveStoreName;
  vector<IncomingMessage> *newMessages;
};

class PhoneListener : public SigC::Object
{
 public:
  PhoneListener();

  bool add_device(std::string devicename);
  void sms_loop(std::string device, Glib::Dispatcher *disp);
  void queue_outgoing_message(Glib::ustring num, Glib::ustring msg);
  void stop();

  // thread creator
  Glib::Thread *sms_loop_thread(std::string device, Glib::Dispatcher *disp);

  // signals
  typedef SigC::Signal3<void, Glib::ustring, Glib::ustring, Glib::ustring> notify_sig_t;
  typedef SigC::Signal1<void, gint> status_sig_t;
  notify_sig_t signal_notify();
  status_sig_t signal_status();

 protected:
  // methods
  void PhoneListener::send_message_notification(gsmlib::SMSMessageRef msg);
  // member variables
  notify_sig_t m_signal_notify;
  status_sig_t m_signal_status;
  bool terminateSent();
 
 private:
  Glib::Mutex mutex;
  bool m_terminateSent;
  bool m_erase_mode;
  vector<IncomingMessage> newMessages;
  vector<OutgoingMessage> sendQueue;

  // methods
  void init();
  void send_message(gsmlib::MeTa *mt, OutgoingMessage *om);
};
#endif
