#include <config.h>
#include <math.h>
#include <gnome.h>
#include <panel-applet.h>
#include <panel-applet-gconf.h>
#include <glade/glade.h>
#include <glibmm.h>
#include <gnome.h>
#include <libgnomeuimm.h>
#include <gconfmm.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "application.h"

static gchar *icon_names[]={
  "cellphone.png",
  "cellphone-connecting.png",
  "cellphone-message.png",
  "cellphone-error.png"
};


typedef struct {
  PanelApplet *applet;
  GtkWidget   *fixed;
  GtkTooltips *tooltips;
  GdkPixbuf   *phone_image[STATE_LAST];
  GdkPixbuf   *phone_image_scaled;
  GtkWidget   *phone_icon;
  PMgrApp     *pmgr;
  GNOME_Bluetooth_Manager btmanager;
  guint       state;
  gint        current_size;
} PhoneMgrApplet;


static void
help_cb (BonoboUIComponent *uic,
		 PhoneMgrApplet    *applet,
		 const char        *verbname)
{
  g_message("help callback");
}

static void
about_cb (BonoboUIComponent *uic,
		  PhoneMgrApplet    *applet,
		  const char        *verbname)
{
    static GtkWidget *about = NULL;
    GdkPixbuf    *pixbuf;
    GError       *error = NULL;

	static const gchar *authors [] = {
	  "Edd Dumbill <edd@usefulinc.com>",
	NULL
    };

    const gchar *documenters[] = {
	NULL
    };

    const gchar *translator_credits = _("translator_credits");

    if (about) {
	gtk_window_present (GTK_WINDOW (about));
	return;
    }

    pixbuf = gdk_pixbuf_new_from_file (ICONDIR"cellphone.png", &error);

	about = gnome_about_new (
	_("Phone Manager"), VERSION,
	_("Copyright (c) 2002-2003 Edd Dumbill"),
	_("Send and receive text messages with your mobile phone."),
	authors,
	documenters,
	strcmp (translator_credits, "translator_credits")
		!= 0 ? translator_credits : NULL,
	pixbuf);

    if (pixbuf)
	g_object_unref (pixbuf);

    gtk_window_set_wmclass (GTK_WINDOW (about), "phonemgr",
							"Phone Manager");

    g_signal_connect (about, "destroy",
	      G_CALLBACK (gtk_widget_destroyed),
	      &about);
	gtk_widget_show (about);
}

static void
properties_cb (BonoboUIComponent *uic,
			   PhoneMgrApplet        *applet,
			   const char        *verbname)
{
  if (applet->pmgr)
	applet->pmgr->show_preferences();
}

static void
send_cb  (BonoboUIComponent *uic,
			   PhoneMgrApplet        *applet,
			   const char        *verbname)
{
  if (applet->pmgr)
	applet->pmgr->create_send_dialog("");
}

// handler called when the fixed layout container in
// the applet is destroyed.
// this isn't actually the best as it means the user
// loses visual feedback while disconnection is being
// performed and just sees a grey box instead.
// TODO: fix that
static void
destroy_cb (GtkObject *object, PhoneMgrApplet *ap)
{
  int i=0;
  g_return_if_fail(ap);
  if (ap->pmgr)
	delete ap->pmgr;

  for(i=0; i<STATE_LAST; i++) {
	g_object_unref(ap->phone_image[i]);
  }

  if (ap->phone_image_scaled)
	g_object_unref(ap->phone_image_scaled);

  if (ap->tooltips)
	g_object_unref(ap->tooltips);

  g_free(ap);
}

static void
size_icon(PhoneMgrApplet *ap, gint size)
{
  GdkPixbuf *old=0;

  if (ap->phone_image_scaled)
	old=ap->phone_image_scaled;

  ap->phone_image_scaled=gdk_pixbuf_scale_simple(ap->phone_image[ap->state],
												 size, size,
												 GDK_INTERP_HYPER);
  gtk_image_set_from_pixbuf((GtkImage*)ap->phone_icon,
							ap->phone_image_scaled);

  if (size!=ap->current_size)
	gtk_widget_set_size_request(ap->phone_icon,
								size, size);

  ap->current_size=size;

  if (old)
	g_object_unref(old);
};

// called by the main application when the main state
// changes -- so we can reflect this in the applet's
// visual representation
static void
on_ui_state_change(guint status, gchar *tooltip, gpointer data)
{
  PhoneMgrApplet *ap=(PhoneMgrApplet*)data;

  ap->state=status;
  size_icon(ap, ap->current_size);

  if (tooltip)
	gtk_tooltips_set_tip(ap->tooltips, GTK_WIDGET(ap->applet),
						 tooltip, NULL);

}

static void
applet_size_change (PanelApplet *a,
					gint size,
					PhoneMgrApplet  *ap)
{
  size_icon(ap, size);
}

static void
applet_back_change (PanelApplet *a,
					PanelAppletBackgroundType  type,
					GdkColor                  *color,
					const gchar               *pixmap,
					PhoneMgrApplet            *ap)
{
  GtkRcStyle *rc_style = gtk_rc_style_new ();

  switch (type) {
  case PANEL_PIXMAP_BACKGROUND:
	break;
  case PANEL_COLOR_BACKGROUND:
	gtk_widget_modify_bg (ap->phone_icon,
						  GTK_STATE_NORMAL,
						  color);
	break;
  default:
	gtk_widget_modify_style(ap->phone_icon, rc_style);
	break;
  }
  return;
}

static const BonoboUIVerb phonemgr_applet_menu_verbs [] = {
  BONOBO_UI_UNSAFE_VERB ("Send", send_cb),
	BONOBO_UI_UNSAFE_VERB ("Props", properties_cb),
	BONOBO_UI_UNSAFE_VERB ("Help", help_cb),
	BONOBO_UI_UNSAFE_VERB ("About", about_cb),

	BONOBO_UI_VERB_END
};

static gboolean
create_main_app(gpointer data)
{
  PhoneMgrApplet *ap=(PhoneMgrApplet*)data;

  // create application and hook it back to us.
  ap->pmgr=new PMgrApp();
  ap->pmgr->set_ui_status_hook(on_ui_state_change, (gpointer)ap);

  // return false as we're a one-shot handler
  return FALSE;
}

static void
set_atk_name_description (GtkWidget *widget, const gchar *name,
    const gchar *description)
{
    AtkObject *aobj;

    aobj = gtk_widget_get_accessible (widget);
    /* Check if gail is loaded */
    if (GTK_IS_ACCESSIBLE (aobj) == FALSE)
	return;

    atk_object_set_name (aobj, name);
    atk_object_set_description (aobj, description);
}


static gboolean
phonemgr_applet_fill (PanelApplet *applet)
{
  PhoneMgrApplet *ap;
  GError *error = NULL;
  gint size;
  int i;

  gnome_window_icon_set_default_from_file(ICONDIR"cellphone.png");

  ap=g_new0(PhoneMgrApplet, 1);
  ap->applet=applet;

  panel_applet_setup_menu_from_file(applet,
									NULL,
									UIDIR"GNOME_PhonemanagerApplet.xml",
									NULL,
									phonemgr_applet_menu_verbs,
									ap);

  ap->fixed  = gtk_fixed_new ();

  gtk_container_add(GTK_CONTAINER(applet), ap->fixed);

  size=panel_applet_get_size(applet);

  for(i=0; i<STATE_LAST; i++) {
	gchar *fname=g_strdup_printf(ICONDIR"%s", icon_names[i]);
	ap->phone_image[i] = gdk_pixbuf_new_from_file(fname, &error);
	g_free(fname);
  }
  ap->phone_icon = gtk_image_new();

  // set state to working while initialization happens
  ap->state=STATE_WORKING;

  ap->tooltips=gtk_tooltips_new();
  g_object_ref(ap->tooltips);
  gtk_object_sink(GTK_OBJECT(ap->tooltips));
  gtk_tooltips_set_tip(ap->tooltips, GTK_WIDGET(applet),
					   _("Phone Manager"), NULL);

  set_atk_name_description(GTK_WIDGET(applet),
						   _("Phone Manager"),
						   _("Manages messaging on your mobile phone"));

  gtk_fixed_put(GTK_FIXED(ap->fixed), ap->phone_icon, 0, 0);

  // hook up cleanup code
  g_signal_connect (ap->fixed,
					"destroy",
					G_CALLBACK (destroy_cb),
					ap);

  g_signal_connect (applet,
					"change-background",
					G_CALLBACK (applet_back_change),
					ap);

  g_signal_connect (applet,
					"change-size",
					G_CALLBACK (applet_size_change),
					ap);

  glade_gnome_init();
  gdk_threads_init();

  size_icon(ap, size);

  gtk_widget_show_all(GTK_WIDGET(applet));

  // add main app initialisation as an idle handler.
  g_idle_add(create_main_app, (gpointer)ap);

  return TRUE;
}

static gboolean
phonemgr_applet_factory (PanelApplet *applet,
	      const gchar *iid,
	      gpointer     data)
{
    gboolean retval = FALSE;

    if (!strcmp (iid, "OAFIID:GNOME_PhonemanagerApplet"))
	retval = phonemgr_applet_fill (applet);

    return retval;
}

int main(int argc, char *argv[])
{

  try {
	Gnome::Conf::init();
	Gnome::Main gnomeMain("Phone Manager", "0.1",
						  Gnome::UI::module_info_get(),
						  argc, argv);

	/*
	 * you would normally expect bonobo initialization here:
	 * but as we're an applet anyway we'll let panel_applet_factory_main
	 * take care of this initialization for us.

	 if (!bonobo_init(&argc, argv))
	 g_error("Couldn't initialize bonobo");

	 bonobo_activate();
	*/

	return panel_applet_factory_main ("OAFIID:GNOME_PhonemanagerApplet_Factory",
									  (panel_applet_get_type()),
									  phonemgr_applet_factory,
									  NULL);
  } catch(const Glib::Error& ex) {
	cout << ex.what() << endl;
  }
}
