
#include <gtkmm/menu.h>
#include <gtkmm/menubar.h>
#include <gdkmm/cursor.h>
#include <libgnomemm/main.h>
#include <libgnomeuimm/about.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>
#include <gtkmm.h>
#include <sstream>

#include <libgnome/gnome-i18n.h>

#include "prefs.h"
#include "application.h"

/*
  The job of the PrefsManager is to manage the GUI/gconf interaction,
  and provide some decoding of the options into stuff that the main
  app can actually use.
*/

PrefsManager::PrefsManager(PMgrApp *mgr, Glib::RefPtr<Gnome::Glade::Xml> ui) :
  m_disable_gconf(false),
  m_busy_cursor(Gdk::WATCH)
{
  m_mgr=mgr;
  m_ui=ui;
  m_refClient=Gnome::Conf::Client::get_default_client();
  m_refClient->add_dir(CONFBASE);;
  init();
}

PrefsManager::~PrefsManager()
{
  if (m_refClient)
  {
    // Remove callbacks
	for(type_listNotifyIDs::iterator iter = m_listNotifyIDs.begin(); 
		iter != m_listNotifyIDs.end(); iter++)
	{
	  guint notify_id = *iter;
	  m_refClient->notify_remove(notify_id);
    }
    m_listNotifyIDs.clear();
  }
}

void
PrefsManager::show()
{
  Gtk::Window *win=0;
  m_ui->get_widget("prefs_window", win);
  win->property_visible().set_value(true);
}

void
PrefsManager::hide()
{
  Gtk::Window *win=0;
  m_ui->get_widget("prefs_window", win);
  win->property_visible().set_value(false);
}

#define S_CONNECT(x, y) m_ui->get_widget(x, but); \
 if (ctype==y) but->set_active(true); else but->set_active(false); \
 but->signal_toggled().connect(SigC::bind<int, Gtk::RadioButton*>(SigC::slot(*this, &PrefsManager::on_conn_port_change), (int)y, but)); \
 m_radiolist[y]=but;

void 
PrefsManager::init()
{
  Gtk::RadioButton *but=0;
  gint ctype=m_refClient->
	get(CONFBASE"/connection_type").get_int();

  m_dispatcher=new Glib::Dispatcher();
  m_dispatcher->connect(SigC::slot(*this, &PrefsManager::on_bt_discover_exit));

  int i=0;
  for(i=0; i<6; i++) {
	gchar *fname=g_strdup_printf(ICONDIR"bluetooth-work-%d.png", i+1);
	a_anim_frames[i]=Gdk::Pixbuf::create_from_file(fname);
	// TODO: catch exceptions thrown here
  }

  m_ui->get_widget("bt_anim_image", m_anim_image);


  set_dependent_widget(CONNECTION_BLUETOOTH,
					   ctype==CONNECTION_BLUETOOTH ? true : false);
  set_dependent_widget(CONNECTION_OTHER,
					   ctype==CONNECTION_OTHER ? true : false);

  if (!m_mgr->get_bluetooth_active()) {
	// disable bluetooth choice
	m_ui->get_widget("btdevice", but);
	but->set_sensitive(false);
  }

  // the S_CONNECT macro both sets up the handler
  // and also sets the widget active if gconf says so
  // and stuffs it in a handy array for later use

  S_CONNECT("btdevice",  CONNECTION_BLUETOOTH);
  S_CONNECT("serialport1", CONNECTION_SERIAL1);
  S_CONNECT("serialport2", CONNECTION_SERIAL2);
  S_CONNECT("irdaport", CONNECTION_IRCOMM);
  S_CONNECT("otherport", CONNECTION_OTHER);

  Gtk::Entry *entry;
  Gtk::OptionMenu *dmenu;
  m_ui->get_widget("btdevlist", dmenu);
  // TODO; do something based on m_refClient->get
  //			  (CONFBASE"/bluetooth_addr").get_string()

  m_ui->get_widget("otherportentry", entry);
  entry->set_text(m_refClient->get
				  (CONFBASE"/other_serial").get_string());

  Gtk::Button *widg=0;
  m_ui->get_widget("prefsapplybutton", widg);
  widg->signal_clicked().connect(SigC::slot(*this, &PrefsManager::apply_prefs));

  m_ui->get_widget("prefsclosebutton", widg);
  widg->signal_clicked().connect(SigC::slot(*this, &PrefsManager::hide));

  m_ui->get_widget("btchoose", widg);
  widg->signal_clicked().connect(SigC::slot(*this, &PrefsManager::update_devices));

  Gtk::CheckButton *check=0;
  m_ui->get_widget("auto_retry", check);
  check->set_active(m_refClient->get
					(CONFBASE"/auto_retry").get_bool());

  // bt device list
  m_btdevice_list=NULL;
  populate_btdevicelist();

  // connection port
  m_conport=NULL;

  // connect up gconf signals
  guint notify_id;
  notify_id=m_refClient->notify_add(CONFBASE"/bluetooth_addr",
									 SigC::slot
									 (*this, 
									  &PrefsManager::on_gconf_bdaddr_change));
  m_listNotifyIDs.push_back(notify_id);
  notify_id=m_refClient->notify_add(CONFBASE"/other_serial",
									 SigC::slot
									 (*this, 
									  &PrefsManager::on_gconf_other_serial_change));
  m_listNotifyIDs.push_back(notify_id);
  notify_id=m_refClient->notify_add(CONFBASE"/auto_retry",
									 SigC::slot
									 (*this, 
									  &PrefsManager::on_gconf_auto_retry_change));
  m_listNotifyIDs.push_back(notify_id);
  notify_id=m_refClient->notify_add(CONFBASE"/connection_type",
									 SigC::slot
									 (*this, 
									  &PrefsManager::on_gconf_port_change));
  m_listNotifyIDs.push_back(notify_id);
}

gchar *
PrefsManager::connect_port()
{
  std::string conport="";
  gchar *retval;

  gint con_type=m_refClient->
	get(CONFBASE"/connection_type").get_int();
  switch(con_type) {
  case CONNECTION_BLUETOOTH:
	{
	  gint portno=m_mgr->m_btmanager->get_rfcomm_port
		(m_refClient->get(CONFBASE"/bluetooth_addr").to_string());
	  if (portno>=0) {
		std::ostringstream s;
		s << "/dev/rfcomm" << portno;
		conport=s.str();
		g_message(_("New bluetooth port is %s"), conport.c_str());
	  } else {
		g_message(_("Unable to obtain RFCOMM connection (%d)"), portno);
	  }
	}
	break;
  case CONNECTION_SERIAL1:
	conport="/dev/ttyS0";
	break;
  case CONNECTION_SERIAL2:
	conport="/dev/ttyS1";
	break;
  case CONNECTION_IRCOMM:
	conport="/dev/ircomm0";
	break;
  case CONNECTION_OTHER:
	conport=m_refClient->get(CONFBASE"/other_serial").
	  to_string();
	break;
  }
  retval=g_strdup(conport.c_str());
  if (!m_conport)
	m_conport=retval;
  return retval;
}

bool
PrefsManager::animate_icon()
{
  m_anim_frame++;
  if (m_anim_frame>5)
	m_anim_frame=0;
  m_anim_image->set(a_anim_frames[m_anim_frame]);

  return true;
}

void
PrefsManager::bt_discover()
{
  m_mgr->m_btmanager->update_device_list();
  (*m_dispatcher)();
}

void
PrefsManager::on_bt_discover_exit()
{
  Gtk::Window *win, *wwin;

  m_discovery_thread->join();
  
  m_ui->get_widget("prefs_window", win);
  m_ui->get_widget("btworkwin", wwin); 
  // termiante animation
  m_timeout_conn.disconnect();
  // hide progress window
  wwin->property_visible().set_value(false);
  populate_btdevicelist(); 

  win->set_sensitive(true);
  // TODO we shouldn't have to call populate_btdevicelist here
  // the bonobo component should set a "last scanned" key time
  // in gconf, which we can listen to.  we know once it's updated
  // then we can reload from gconf: this will enable us to ensure that the
  // prefs update if somebody else causes a rescan
}

void 
PrefsManager::update_devices()
{
  Gtk::Window *win, *wwin;
  m_ui->get_widget("prefs_window", win);
  m_ui->get_widget("btworkwin", wwin); 
 
  m_anim_frame=0;
  m_anim_image->set(a_anim_frames[m_anim_frame]);
  wwin->property_visible().set_value(true);

  // set timeout to animate icon
  m_timeout_conn=Glib::signal_timeout().connect(SigC::slot(*this, 
														   &PrefsManager::animate_icon),
												200);
  win->set_sensitive(false);

  m_discovery_thread=Glib::Thread::create
	(SigC::slot(*this, &PrefsManager::bt_discover), true);
}

bool
PrefsManager::auto_retry()
{
  return m_refClient->get(CONFBASE"/auto_retry").get_bool();
}

PrefsManager::serial_sig_t
PrefsManager::signal_serial()
{
  return m_signal_serial;
}


void
PrefsManager::set_dependent_widget(gint conn_type, bool active)
{
  Gtk::Widget *widg=0;
  switch(conn_type) {
  case CONNECTION_BLUETOOTH:
	active=active && m_mgr->get_bluetooth_active();
	m_ui->get_widget("btdevlist", widg);
	widg->set_sensitive(active && m_devices_available);
	m_ui->get_widget("btchoose", widg);
	widg->set_sensitive(active);
	break;
  case CONNECTION_OTHER:
	m_ui->get_widget("otherportentry", widg);
	widg->set_sensitive(active);
	break;
  }
}

void
PrefsManager::on_conn_port_change(gint conn_type, Gtk::RadioButton *but)
{
  bool active=but->get_active();	

  set_dependent_widget(conn_type, active);
}

void
PrefsManager::on_gconf_port_change(guint cnxn_id, 
										Gnome::Conf::Entry conf_entry)
{
  gint conn_type=conf_entry.get_value().get_int();
  if (!m_disable_gconf) {
	if (conn_type>=0 && conn_type<CONNECTION_LAST) {
	  m_radiolist[conn_type]->set_active();
	}
  }
}

void
PrefsManager::on_gconf_auto_retry_change(guint cnxn_id, 
										 Gnome::Conf::Entry conf_entry)
{
  Gtk::CheckButton *but=0;
  if (!m_disable_gconf) {
	m_ui->get_widget("auto_retry", but);
	but->set_active(conf_entry.get_value().get_bool());
  }
}

void
PrefsManager::on_gconf_bdaddr_change(guint cnxn_id, Gnome::Conf::Entry conf_entry)
{
  if (!m_disable_gconf) {
	populate_btdevicelist();
  }
}

void
PrefsManager::on_gconf_other_serial_change(guint cnxn_id, 
										   Gnome::Conf::Entry conf_entry)
{
  Gtk::Entry *entry=0;
  if (!m_disable_gconf) {
	m_ui->get_widget("otherportentry", entry);
	entry->set_text(conf_entry.get_value().to_string());
  }
}

void
PrefsManager::apply_prefs()
{
  int conn_type=0;
  int i=0;
  gchar *newport;

  // time to save prefs to gconf
  for (i=0; i<CONNECTION_LAST; i++) {
	if (m_radiolist[i]->get_active()) {
	  conn_type=i;
	}
  }
  m_disable_gconf=true;

  Gtk::Entry *entry=0;

  m_refClient->set(CONFBASE"/bluetooth_addr", get_current_bdaddr());

  m_ui->get_widget("otherportentry", entry);
  m_refClient->set(CONFBASE"/other_serial", entry->get_text());

  Gtk::CheckButton *but=0;
  m_ui->get_widget("auto_retry", but);
  m_refClient->set(CONFBASE"/auto_retry", but->get_active());
 
  // do connection type last
  m_refClient->set(CONFBASE"/connection_type", conn_type);
  m_disable_gconf=false;
  
  newport=connect_port();

  if (!m_conport || g_strcasecmp(newport, m_conport)!=0) {
	// change
	if (m_conport)
	  g_free(m_conport);
	m_conport=newport;
	m_signal_serial.emit(newport);
  } else {
	// no change, so throw away dupe value
	g_free(newport);
  }
}

Glib::ustring
PrefsManager::get_current_bdaddr()
{
  Gtk::OptionMenu *dmenu=0;
  m_ui->get_widget("btdevlist", dmenu);
  Gtk::Menu::MenuList& menulist=dmenu->get_menu()->items();
  Gtk::MenuItem *item;

  if (m_devices_available) {
	item=&menulist[dmenu->get_history()];
	return ((BluetoothDeviceItem*)item)->get_btdevice().get_bdaddr();
  } else {
	return "00:00:00:00:00:00";
  }
}

void
PrefsManager::populate_btdevicelist()
{
  Gtk::Menu *menu;
  Gtk::OptionMenu *optmen;
  std::list <BluetoothDeviceItem*> *oldlist;
  std::list<BluetoothDeviceItem*>::iterator i;

  const std::string &cur_bdaddr=m_refClient->get(CONFBASE"/bluetooth_addr").to_string();

  m_ui->get_widget("btdevicemenu", menu);
  m_ui->get_widget("btdevlist", optmen);
  Gtk::Menu::MenuList& menulist=menu->items();

  if (m_mgr->get_bluetooth_active()) {
	oldlist=m_btdevice_list;
	m_btdevice_list=m_mgr->m_btmanager->get_device_list();

	if (oldlist) {
	  i=oldlist->begin();
	  while(i!=oldlist->end()) {
		menu->remove(**i);
		i++;
	  }
	  m_mgr->m_btmanager->free_device_list(oldlist);
	} 

	// TODO get rid of iterator crud, it's too complex
	int n=0;
	optmen->set_history(0);
	i=m_btdevice_list->begin();
	while(i!=m_btdevice_list->end()) {
	  menulist.push_back(**i);
	  if ((*i)->get_btdevice().get_bdaddr() == cur_bdaddr) {
		optmen->set_history(n);
	  }
	  i++; n++;
	}
	if (n>0) {
	  m_devices_available=true;
	} else {
	  m_devices_available=false;
	}
	optmen->set_sensitive(m_devices_available);
  }
}
