#ifndef _PREFS_H
#define _PREFS_H

#include <gtkmm/label.h>
#include <gtkmm/button.h>
#include <gtkmm/frame.h>
#include <gtkmm/image.h>
#include <gtkmm/main.h>
#include <gdkmm/cursor.h>
#include <libgnomeuimm/app.h>
#include <libgnomeuimm/appbar.h>
#include <libgnomeuimm/about.h>
#include <libglademm.h>
#include <gconfmm.h>
#include <vector>
#include <queue>
#include <list>

#include "btmanager.h"

#define CONFBASE "/apps/phonemgr/prefs"

enum {
  CONNECTION_BLUETOOTH,
  CONNECTION_SERIAL1,
  CONNECTION_SERIAL2,
  CONNECTION_IRCOMM,
  CONNECTION_OTHER,
  CONNECTION_LAST
};

class PrefsManager;
class PMgrApp;

class PrefsManager : public SigC::Object
{
 public:
  PrefsManager(PMgrApp *mgr, Glib::RefPtr<Gnome::Glade::Xml> ui);
  ~PrefsManager();
  void show();
  void hide();

  // which port to connect to 
  gchar *connect_port();

  // whether to retry connections
  bool auto_retry();

  // gui signal handlers
  void on_conn_port_change(gint, Gtk::RadioButton*);
  void apply_prefs();

  // gconf signal handlers
  void on_gconf_port_change(guint cnxn_id, Gnome::Conf::Entry conf_entry);
  void on_gconf_auto_retry_change(guint cnxn_id, Gnome::Conf::Entry conf_entry);
  void on_gconf_bdaddr_change(guint cnxn_id, Gnome::Conf::Entry conf_entry);
  void on_gconf_other_serial_change(guint cnxn_id, Gnome::Conf::Entry conf_entry);

  // signals
  typedef SigC::Signal1<void, Glib::ustring> serial_sig_t;
  serial_sig_t signal_serial();

 protected:
  void init();
  void populate_btdevicelist();
  void update_devices();
  void show_working_win();
  void bt_discover();
  void on_bt_discover_exit();
  bool animate_icon();
  PMgrApp *m_mgr;
  Glib::RefPtr<Gnome::Glade::Xml> m_ui;
  Glib::RefPtr<Gnome::Conf::Client> m_refClient;
  void set_dependent_widget(gint c, bool active);
  typedef std::list<guint> type_listNotifyIDs;
  // callback connection IDs from GConf::Client::notify_add()
  std::list<guint> m_listNotifyIDs; 
  serial_sig_t m_signal_serial;
  Gtk::RadioButton *m_radiolist[CONNECTION_LAST];
  bool m_disable_gconf;
  bool m_devices_available;
  std::list<BluetoothDeviceItem *> *m_btdevice_list;
  Glib::ustring get_current_bdaddr();
  gchar *m_conport;
  Gdk::Cursor::Cursor m_busy_cursor;
  Glib::RefPtr<Gdk::Pixbuf> a_anim_frames[6];
  Gtk::Image *m_anim_image;
  gint m_anim_frame;
  Glib::Dispatcher *m_dispatcher;
  SigC::Connection m_timeout_conn;
  Glib::Thread *m_discovery_thread;
};
#endif
